@@ -4,7 +4,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -27,7 +27,6 @@
 
 #ifdef HAVE_RL_GETC_FUNCTION
 #include "libguile.h"
-#include "libguile/gh.h"
 #include "libguile/iselect.h"
 
 #include <stdio.h>
@@ -52,7 +51,8 @@ scm_t_option scm_readline_opts[] = {
   { SCM_OPTION_INTEGER, "history-length", 200,
     "History length." },
   { SCM_OPTION_INTEGER, "bounce-parens", 500,
-    "Time (ms) to show matching opening parenthesis (0 = off)."}
+    "Time (ms) to show matching opening parenthesis (0 = off)."},
+  { 0 }
 };
 
 extern void stifle_history (int max);
@@ -64,7 +64,6 @@ SCM_DEFINE (scm_readline_options, "readline-options-interface", 0, 1, 0,
 {
   SCM ans = scm_options (setting,
 			 scm_readline_opts,
-			 SCM_N_READLINE_OPTIONS,
 			 FUNC_NAME);
   stifle_history (SCM_HISTORY_LENGTH);
   return ans;
@@ -576,8 +575,7 @@ scm_init_readline ()
 
   reentry_barrier_mutex = scm_permanent_object (scm_make_mutex ());
   scm_init_opts (scm_readline_options,
-		 scm_readline_opts,
-		 SCM_N_READLINE_OPTIONS);
+		 scm_readline_opts);
 #if HAVE_RL_GET_KEYMAP
   init_bouncing_parens();
 #endif
