@@ -6,18 +6,19 @@
 /* Copyright (C) 1995,1996,2000,2001, 2002, 2006, 2008 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
@@ -38,6 +39,25 @@
 #  include <config.h>
 #endif
 
+/* Undefine HAVE_STRUCT_TIMESPEC, because the libguile C code doesn't
+   need it anymore, and because on MinGW:
+
+   - the definition of struct timespec is provided (if at all) by
+     pthread.h
+
+   - pthread.h will _not_ define struct timespec if
+     HAVE_STRUCT_TIMESPEC is 1, because then it thinks that it doesn't
+     need to.
+
+   The libguile C code doesn't need HAVE_STRUCT_TIMESPEC anymore,
+   because the value of HAVE_STRUCT_TIMESPEC has already been
+   incorporated in how scm_t_timespec is defined (in scmconfig.h), and
+   the rest of the libguile C code now just uses scm_t_timespec.
+ */
+#ifdef HAVE_STRUCT_TIMESPEC
+#undef HAVE_STRUCT_TIMESPEC
+#endif
+
 #include <errno.h>
 #include "libguile/__scm.h"
 
