@@ -1,25 +1,26 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2004, 2006, 2008 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
 
 #define _LARGEFILE64_SOURCE      /* ask for stat64 etc */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
@@ -31,6 +32,7 @@
 #include "libguile/gc.h"
 #include "libguile/posix.h"
 #include "libguile/dynwind.h"
+#include "libguile/hashtab.h"
 
 #include "libguile/fports.h"
 
@@ -58,6 +60,8 @@
 # include <winsock2.h>
 #endif /* __MINGW32__ */
 
+#include <full-write.h>
+
 /* Mingw (version 3.4.5, circa 2006) has ftruncate as an alias for chsize
    already, but have this code here in case that wasn't so in past versions,
    or perhaps to help other minimal DOS environments.
@@ -220,33 +224,29 @@ SCM_DEFINE (scm_setvbuf, "setvbuf", 2, 1, 0,
 /* Move ports with the specified file descriptor to new descriptors,
  * resetting the revealed count to 0.
  */
+static void
+scm_i_evict_port (void *closure, SCM port)
+{
+  int fd = * (int*) closure;
+
+  if (SCM_FPORTP (port))
+    {
+      scm_t_fport *fp = SCM_FSTREAM (port);
+
+      if (fp->fdes == fd)
+	{
+	  fp->fdes = dup (fd);
+	  if (fp->fdes == -1)
+	    scm_syserror ("scm_evict_ports");
+	  scm_set_port_revealed_x (port, scm_from_int (0));
+	}
+    }
+}
 
 void
 scm_evict_ports (int fd)
 {
-  long i;
-
-  scm_i_scm_pthread_mutex_lock (&scm_i_port_table_mutex);
-
-  for (i = 0; i < scm_i_port_table_size; i++)
-    {
-      SCM port = scm_i_port_table[i]->port;
-
-      if (SCM_FPORTP (port))
-	{
-	  scm_t_fport *fp = SCM_FSTREAM (port);
-
-	  if (fp->fdes == fd)
-	    {
-	      fp->fdes = dup (fd);
-	      if (fp->fdes == -1)
-		scm_syserror ("scm_evict_ports");
-	      scm_set_port_revealed_x (port, scm_from_int (0));
-	    }
-	}
-    }
-
-  scm_i_pthread_mutex_unlock (&scm_i_port_table_mutex);
+  scm_c_port_for_each (scm_i_evict_port, (void *) &fd);
 }
 
 
@@ -829,9 +829,9 @@ fport_flush (SCM port)
 	      const char *msg = "Error: could not flush file-descriptor ";
 	      char buf[11];
 
-	      write (2, msg, strlen (msg));
+	      full_write (2, msg, strlen (msg));
 	      sprintf (buf, "%d\n", fp->fdes);
-	      write (2, buf, strlen (buf));
+	      full_write (2, buf, strlen (buf));
 
 	      count = remaining;
 	    }
