@@ -3,21 +3,22 @@
 #ifndef SCM_PRINT_H
 #define SCM_PRINT_H
 
-/* Copyright (C) 1995,1996,1998,2000,2001, 2003, 2004 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,2000,2001, 2003, 2004, 2006, 2008 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
@@ -26,13 +27,6 @@
 
 #include "libguile/options.h"
 
-SCM_API scm_t_option scm_print_opts[];
-
-#define SCM_PRINT_CLOSURE	    (SCM_PACK (scm_print_opts[0].val))
-#define SCM_PRINT_SOURCE_P	    ((int) scm_print_opts[1].val)
-#define SCM_PRINT_HIGHLIGHT_PREFIX  (SCM_PACK (scm_print_opts[2].val))
-#define SCM_PRINT_HIGHLIGHT_SUFFIX  (SCM_PACK (scm_print_opts[3].val))
-#define SCM_N_PRINT_OPTIONS 4
 
 /* State information passed around during printing.
  */
@@ -82,7 +76,7 @@ SCM_API scm_t_bits scm_tc16_port_with_ps;
 SCM_API SCM scm_print_options (SCM setting);
 SCM_API SCM scm_make_print_state (void);
 SCM_API void scm_free_print_state (SCM print_state);
-SCM scm_i_port_with_print_state (SCM port, SCM print_state);
+SCM_INTERNAL SCM scm_i_port_with_print_state (SCM port, SCM print_state);
 SCM_API void scm_intprint (scm_t_intmax n, int radix, SCM port);
 SCM_API void scm_uintprint (scm_t_uintmax n, int radix, SCM port);
 SCM_API void scm_ipruk (char *hdr, SCM ptr, SCM port);
@@ -99,7 +93,7 @@ SCM_API SCM scm_printer_apply (SCM proc, SCM exp, SCM port, scm_print_state *);
 SCM_API SCM scm_port_with_print_state (SCM port, SCM pstate);
 SCM_API SCM scm_get_print_state (SCM port);
 SCM_API int scm_valid_oport_value_p (SCM val);
-SCM_API void scm_init_print (void);
+SCM_INTERNAL void scm_init_print (void);
 
 #ifdef GUILE_DEBUG
 SCM_API SCM scm_current_pstate (void);
